/*
 * FILE:    NSmain.java
 * AUTHORS: Kristian Hasler
 * 
 * Copyright (c) 1999 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted, for non-commercial use only, provided
 * that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Computer Science
 *      Department at University College London
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * Use of this software for commercial purposes is explicitly forbidden
 * unless prior written permission is obtained from the authors.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

import java.awt.*;
import java.applet.*;
import java.util.*;
import java.io.*;
import netscape.security.PrivilegeManager;

public class NSmain extends Applet implements Runnable
{
	private GUI gui;
	private ParseSDP sdp;
    private separate plugins;

	protected void startTool (String commandline[]) {
		if (commandline.equals(null)) return;
		PrivilegeManager.enablePrivilege("UniversalExecAccess");

		Runtime r = Runtime.getRuntime();
		Process rec;
		showStatus ("Starting "+commandline[0]);
		try {
			rec = r.exec (commandline);
		} catch (Exception e) {
			System.out.println(e);
			showStatus ("Error executing "+commandline[0]);
		}

	}

    public void init() {
		System.out.println("-----------------------------------------------------------");
		System.out.println("Sdp Parser (SPar) version 1.2");
		System.out.println("Web: http://www-mice.cs.ucl.ac.uk/multimedia/software/spar/");
		System.out.println("Author: Kristian Hasler");
		System.out.println("Copyright: (c) 1999 University College London");
		System.out.println("-----------------------------------------------------------");
		System.out.println("Operating System Name: "+System.getProperty("os.name"));

		try {
			PrivilegeManager.enablePrivilege("UniversalExecAccess");
            //PrivilegeManager.enablePrivilege("UniversalFileRead");
            //PrivilegeManager.enablePrivilege("UniversalPropertyRead");
		} catch (netscape.security.ForbiddenTargetException e) {
			showStatus("Failed! Permission to execute programs denied by user.");
			return;
		} catch (Exception e) {
			showStatus("Failed! Unknown exception while enabling privilege.");
			e.printStackTrace(System.out);
			return;
		}
		System.out.println("Permission to execute programs granted by user.");

		gui=new GUI();
		setBackground(Color.white);
		add ("Center",gui);
		
		sdp=new	ParseSDP (getParameter("SDP"), getParameter("Separator"), gui);
        plugins=new separate(getParameter("Plugins"), getParameter("Separator"));
        for (int media=0; media<sdp.tool+1; media++) {
            checkTools (media);
        }
	}

	public void run() {
	}

	public boolean handleEvent(Event event) {
        String[] commandLine;
        boolean duplicate=false;
        String executable, flag;
        Hashtable flagTable;
        Stack flagStack;

		switch (event.id) {
		case Event.ACTION_EVENT:
			for (int i=0; i<sdp.tool+1; i++) {
                /* if the same tools exist for multiple media types then select them */
                if (event.target==gui.choice[i]) {
                    for (int j=0; j<sdp.tool+1; j++) {
                        gui.selectTool(j, gui.getTool(i));
                        if (gui.getTool(j).equals(gui.getTool(i))) gui.selectMedia(j);
                    }
                }
			}
			if (event.target==gui.startButton) {
                /* take each media and if it's selected, get the executable */
				for (int media=0; media<sdp.tool+1; media++) {
                    if (gui.mediaSelected(media)) {
                        flagTable=new Hashtable(20);
                        executable=gui.getTool(media);
                        /* get all media types that have this executable selected */
                        for (int j=0; j<sdp.tool+1; j++) {
                            /* find the plugin for each media and put flags on stack */
                            if ((executable.equals(gui.getTool(j))) && (gui.mediaSelected(j))) {
                                /* if we are starting this tool we can deselect the media */
                                gui.deselectMedia(j);
                                flagStack=new Stack();
                                flagStack=sdp.parsePlugins(j, executable, plugins);
                                for (int flagNo=0; (!flagStack.empty()); flagNo++) {
                                    flag=(flagStack.pop().toString());
                                    /* check for duplicates in flag table */
                                    for (int flagTableNo=0; flagTableNo<flagTable.size(); flagTableNo++) {
                                        if (flagTable.get(Integer.toString(flagTableNo)).equals(flag)) {
                                                duplicate=true;
                                                break;
                                        } else duplicate=false;
                                    }
                                    /* no duplicate, so we add it to the end */
                                    if (!duplicate) flagTable.put(Integer.toString(flagTable.size()), flag);
                                }
                            }
                        }
                    
                        /* copy table into commandline string array */
                        commandLine=new String[flagTable.size()+1];
                        commandLine[0]=executable;
                        for (int i=0; i<flagTable.size(); i++) {
                            commandLine[flagTable.size()-i]=(flagTable.get(Integer.toString(i)).toString());
                        }
                        System.out.println("command line:");
		                for (int i=0; i<commandLine.length; i++) System.out.println(commandLine[i]);
					    startTool(commandLine);
                    }
				}
			}
		break;
		}
	return true;
	}

    protected void checkTools(int media) {
		int lineNo=0;
        boolean foundformat=false, foundmedia=false, foundtool=false;
        String toolfound="";
		// scan through lines
        System.out.println("checking plugin for media: "+sdp.media[media]+" and format: "+sdp.format[media]);
		while (lineNo<plugins.numLines()) {
			if (plugins.line(lineNo).startsWith("media:")) {
				if ((plugins.line(lineNo).substring(6)).equals(sdp.media[media])) {
                    System.out.println ("found media: "+sdp.media[media]);
                    foundmedia=true;
                    foundformat=false;
                }
			}
            if (plugins.line(lineNo).startsWith("fmt:")) {
				if ((plugins.line(lineNo).substring(4)).equals(sdp.format[media])) {
                    System.out.println ("found format: "+sdp.format[media]);
                    foundformat=true;
                }
			}
            if (plugins.line(lineNo).startsWith("tool:")) {
                foundtool=true;
                toolfound=plugins.line(lineNo).substring(5);
            }
            if (foundmedia && foundformat && foundtool) {
                System.out.println ("adding tool: "+toolfound);
                //PrivilegeManager.enablePrivilege("UniversalFileRead");
                //PrivilegeManager.enablePrivilege("UniversalPropertyRead");
                gui.addTool(media, toolfound);
                foundmedia=foundformat=foundtool=false;
            }
            lineNo++;
        }
    }

}
